@@ -5,7 +5,7 @@ from functools import lru_cache
 import sys, locale, codecs, os, collections, collections.abc
 
 __appname__   = 'calibre'
-numeric_version = (5, 42, 0)
+numeric_version = (6, 2, 1)
 __version__   = '.'.join(map(str, numeric_version))
 git_version   = None
 __author__    = "Kovid Goyal <kovid@kovidgoyal.net>"
@@ -34,11 +34,12 @@ if iswindows:
     wver = sys.getwindowsversion()
     isxp = wver.major < 6
     isoldvista = wver.build < 6002
-is64bit = sys.maxsize > (1 << 32)
+is64bit = True
 isworker = hasenv('CALIBRE_WORKER') or hasenv('CALIBRE_SIMPLE_WORKER')
 if isworker:
     os.environ.pop(environ_item('CALIBRE_FORCE_ANSI'), None)
 FAKE_PROTOCOL, FAKE_HOST = 'clbr', 'internal.invalid'
+SPECIAL_TITLE_FOR_WEBENGINE_COMMS = '__webengine_messages_pending__'
 VIEWER_APP_UID = 'com.calibre-ebook.viewer'
 EDITOR_APP_UID = 'com.calibre-ebook.edit-book'
 MAIN_APP_UID = 'com.calibre-ebook.main-gui'
@@ -108,9 +109,13 @@ else:
 DEBUG = hasenv('CALIBRE_DEBUG')
 
 
-def debug():
+def debug(val=True):
     global DEBUG
-    DEBUG = True
+    DEBUG = bool(val)
+
+
+def is_debugging():
+    return DEBUG
 
 
 def _get_cache_dir():
@@ -199,6 +204,15 @@ class DeVendor:
             return find_spec('feedparser')
         if fullname.startswith('calibre.ebooks.markdown'):
             return ModuleSpec(fullname, DeVendorLoader(fullname[len('calibre.ebooks.'):]))
+        if fullname.startswith('PyQt5'):
+            # this is present for third party plugin compat
+            if fullname == 'PyQt5':
+                return ModuleSpec(fullname, DeVendorLoader('qt'))
+            return ModuleSpec(fullname, DeVendorLoader('qt.webengine' if 'QWebEngine' in fullname else 'qt.core'))
+        if fullname.startswith('Cryptodome'):
+            # this is needed for py7zr which uses pycryptodomex instead of
+            # pycryptodome for some reason
+            return ModuleSpec(fullname, DeVendorLoader(fullname.replace('dome', '', 1)))
 
 
 class ExtensionsPackageLoader:
@@ -235,6 +249,7 @@ class ExtensionsImporter:
             'podofo',
             'cPalmdoc',
             'progress_indicator',
+            'rcc_backend',
             'icu',
             'speedup',
             'html_as_json',
@@ -251,6 +266,7 @@ class ExtensionsImporter:
             'tokenizer',
             'certgen',
             'sqlite_extension',
+            'uchardet',
         )
         if iswindows:
             extra = ('winutil', 'wpd', 'winfonts', 'winsapi')
@@ -330,6 +346,15 @@ class Plugins(collections.abc.Mapping):
         finally:
             conn.enableloadextension(False)
 
+    def load_sqlite3_extension(self, conn, name):
+        conn.enable_load_extension(True)
+        try:
+            ext = 'pyd' if iswindows else 'so'
+            path = os.path.join(plugins_loc, f'{name}.{ext}')
+            conn.load_extension(path)
+        finally:
+            conn.enable_load_extension(False)
+
 
 plugins = None
 if plugins is None:
@@ -387,6 +412,9 @@ if getattr(sys, 'frozen', False):
         is_running_from_develop = running_in_develop_mode()
 
 in_develop_mode = os.getenv('CALIBRE_ENABLE_DEVELOP_MODE') == '1'
+if iswindows:
+    # Needed to get Qt to use the correct cache dir, relies on a patched Qt
+    os.environ['CALIBRE_QT_CACHE_LOCATION'] = cache_dir()
 
 
 def get_version():
@@ -399,8 +427,6 @@ def get_version():
             v = v[:-2]
     if is_running_from_develop:
         v += '*'
-    if iswindows and is64bit:
-        v += ' [64bit]'
 
     return v
 
@@ -455,7 +481,7 @@ def get_umask():
     return mask
 
 
-# call this at startup as it changed process global state, which doesnt work
+# call this at startup as it changed process global state, which doesn't work
 # with multi-threading. It's absurd there is no way to safely read the current
 # umask of a process.
 get_umask()
