@@ -75,11 +75,11 @@ class Configure(Dialog):
         v = QVBoxLayout()
         self.mub = b = QToolButton(self)
         connect_lambda(b.clicked, self, lambda self: move_field_up(fdo, self.model))
-        b.setIcon(QIcon(I('arrow-up.png')))
+        b.setIcon(QIcon.ic('arrow-up.png'))
         b.setToolTip(_('Move the selected field up'))
         v.addWidget(b), v.addStretch(10)
         self.mud = b = QToolButton(self)
-        b.setIcon(QIcon(I('arrow-down.png')))
+        b.setIcon(QIcon.ic('arrow-down.png'))
         b.setToolTip(_('Move the selected field down'))
         connect_lambda(b.clicked, self, lambda self: move_field_down(fdo, self.model))
         v.addWidget(b)
@@ -176,10 +176,10 @@ class BookInfo(QDialog):
             _('Configure this view'), _('Configure')))
         self.clabel.linkActivated.connect(self.configure)
         hl.addWidget(self.clabel)
-        self.previous_button = QPushButton(QIcon(I('previous.png')), _('&Previous'), self)
+        self.previous_button = QPushButton(QIcon.ic('previous.png'), _('&Previous'), self)
         self.previous_button.clicked.connect(self.previous)
         l2.addWidget(self.previous_button, l2.rowCount(), 0)
-        self.next_button = QPushButton(QIcon(I('next.png')), _('&Next'), self)
+        self.next_button = QPushButton(QIcon.ic('next.png'), _('&Next'), self)
         self.next_button.clicked.connect(self.next)
         l2.addWidget(self.next_button, l2.rowCount() - 1, 1)
 
@@ -209,12 +209,21 @@ class BookInfo(QDialog):
                 self.splitter.restoreState(saved_layout[1])
             except Exception:
                 pass
-        from calibre.gui2.ui import get_gui
         ema = get_gui().iactions['Edit Metadata'].menuless_qaction
         a = self.ema = QAction('edit metadata', self)
         a.setShortcut(ema.shortcut())
         self.addAction(a)
         a.triggered.connect(self.edit_metadata)
+        vb = get_gui().iactions['View'].menuless_qaction
+        a = self.vba = QAction('view book', self)
+        a.setShortcut(vb.shortcut())
+        a.triggered.connect(self.view_book)
+        self.addAction(a)
+
+    def view_book(self):
+        if self.current_row is not None:
+            book_id = self.view.model().id(self.current_row)
+            get_gui().iactions['View']._view_calibre_books((book_id,))
 
     def edit_metadata(self):
         if self.current_row is not None:
