@@ -46,7 +46,7 @@ class DBCheck(QDialog):  # {{{
         t.daemon = True
         t.start()
         QTimer.singleShot(100, self.check)
-        self.exec_()
+        self.exec()
 
     def vacuum(self):
         try:
@@ -87,7 +87,7 @@ class CheckLibraryDialog(QDialog):
         self.db = db
 
         self.setWindowTitle(_('Check library -- Problems found'))
-        self.setWindowIcon(QIcon(I('debug.png')))
+        self.setWindowIcon(QIcon.ic('debug.png'))
 
         self._tl = QHBoxLayout()
         self.setLayout(self._tl)
@@ -233,7 +233,7 @@ class CheckLibraryDialog(QDialog):
             probs += self.problem_count[c]
         if probs == 0:
             return False
-        self.exec_()
+        self.exec()
         return True
 
     def accept(self):
@@ -266,25 +266,25 @@ class CheckLibraryDialog(QDialog):
                 tl.setData(1, Qt.ItemDataRole.UserRole, self.is_fixable)
                 tl.setText(1, _('(fixable)'))
                 tl.setFlags(Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsUserCheckable)
-                tl.setCheckState(1, False)
+                tl.setCheckState(1, Qt.CheckState.Unchecked)
             else:
                 tl.setData(1, Qt.ItemDataRole.UserRole, self.is_deletable)
                 tl.setData(2, Qt.ItemDataRole.UserRole, self.is_deletable)
                 tl.setText(1, _('(deletable)'))
                 tl.setFlags(Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsUserCheckable)
-                tl.setCheckState(1, False)
+                tl.setCheckState(1, Qt.CheckState.Unchecked)
             if attr == 'extra_covers':
                 tl.setData(2, Qt.ItemDataRole.UserRole, self.is_deletable)
                 tl.setText(2, _('(deletable)'))
                 tl.setFlags(Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsUserCheckable)
-                tl.setCheckState(2, False)
+                tl.setCheckState(2, Qt.CheckState.Unchecked)
             self.top_level_items[attr] = tl
 
             for problem in list_:
                 it = Item()
                 if checkable:
                     it.setFlags(Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsUserCheckable)
-                    it.setCheckState(2, False)
+                    it.setCheckState(2, Qt.CheckState.Unchecked)
                     it.setData(2, Qt.ItemDataRole.UserRole, self.is_deletable)
                 else:
                     it.setFlags(Qt.ItemFlag.ItemIsEnabled)
@@ -318,6 +318,8 @@ class CheckLibraryDialog(QDialog):
 
     def item_changed(self, item, column):
         def set_delete_boxes(node, col, to_what):
+            if isinstance(to_what, bool):
+                to_what = Qt.CheckState.Checked if to_what else Qt.CheckState.Unchecked
             self.log.blockSignals(True)
             if col:
                 node.setCheckState(col, to_what)
@@ -357,7 +359,7 @@ class CheckLibraryDialog(QDialog):
                     set_delete_boxes(item, column, item.checkState(column))
                     if column == 2:
                         self.log.blockSignals(True)
-                        item.setCheckState(1, False)
+                        item.setCheckState(1, Qt.CheckState.Unchecked)
                         self.log.blockSignals(False)
             else:
                 item.setCheckState(column, Qt.CheckState.Unchecked)
@@ -453,7 +455,7 @@ class CheckLibraryDialog(QDialog):
             attr = check[0]
             fixable = check[3]
             tl = self.top_level_items[attr]
-            if fixable and tl.checkState(1):
+            if fixable and tl.checkState(1) == Qt.CheckState.Checked:
                 func = getattr(self, 'fix_' + attr, None)
                 if func is not None and callable(func):
                     func()
@@ -467,4 +469,4 @@ if __name__ == '__main__':
     app = QApplication([])
     from calibre.library import db
     d = CheckLibraryDialog(None, db())
-    d.exec_()
+    d.exec()
