@@ -8,7 +8,7 @@ __docformat__ = 'restructuredtext en'
 from qt.core import (
     QApplication, QCheckBox, QDialog, QDialogButtonBox, QHBoxLayout, QIcon, QImage,
     QLabel, QPainter, QPalette, QPixmap, QScrollArea, QSize, QSizePolicy,
-    QSvgRenderer, Qt, QTransform, QUrl, QVBoxLayout, pyqtSignal, QAction, QKeySequence
+    Qt, QTransform, QUrl, QVBoxLayout, pyqtSignal, QAction, QKeySequence
 )
 
 from calibre import fit_image
@@ -18,6 +18,7 @@ from calibre.gui2 import (
 
 
 def render_svg(widget, path):
+    from qt.core import QSvgRenderer
     img = QPixmap()
     rend = QSvgRenderer()
     if rend.load(path):
@@ -105,6 +106,8 @@ class ScrollArea(QScrollArea):
                 self.zoom_requested.emit(self.current_wheel_angle_delta < 0)
                 self.current_wheel_angle_delta = 0
             ev.accept()
+        else:
+            super().wheelEvent(ev)
 
 
 class ImageView(QDialog):
@@ -139,12 +142,12 @@ class ImageView(QDialog):
         self.copy_button = co = bb.addButton(_('&Copy'), QDialogButtonBox.ButtonRole.ActionRole)
         self.rotate_button = ro = bb.addButton(_('&Rotate'), QDialogButtonBox.ButtonRole.ActionRole)
         self.fullscreen_button = fo = bb.addButton(_('&Full screen'), QDialogButtonBox.ButtonRole.ActionRole)
-        zi.setIcon(QIcon(I('plus.png')))
-        zo.setIcon(QIcon(I('minus.png')))
-        so.setIcon(QIcon(I('save.png')))
+        zi.setIcon(QIcon.ic('plus.png'))
+        zo.setIcon(QIcon.ic('minus.png'))
+        so.setIcon(QIcon.ic('save.png'))
         co.setIcon(QIcon.ic('edit-copy.png'))
-        ro.setIcon(QIcon(I('rotate-right.png')))
-        fo.setIcon(QIcon(I('page.png')))
+        ro.setIcon(QIcon.ic('rotate-right.png'))
+        fo.setIcon(QIcon.ic('page.png'))
         zi.clicked.connect(self.zoom_in)
         zo.clicked.connect(self.zoom_out)
         so.clicked.connect(self.save_image)
