@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
@@ -7,13 +6,12 @@ __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import json
-
 from collections import defaultdict
 from threading import Thread
 
 from qt.core import (
     QApplication, QFont, QFontInfo, QFontDialog, QColorDialog, QPainter, QDialog,
-    QAbstractListModel, Qt, QIcon, QKeySequence, QColor, pyqtSignal, QCursor,
+    QAbstractListModel, Qt, QIcon, QKeySequence, QColor, pyqtSignal, QHeaderView, QListWidgetItem,
     QWidget, QSizePolicy, QBrush, QPixmap, QSize, QPushButton, QVBoxLayout, QItemSelectionModel,
     QTableWidget, QTableWidgetItem, QLabel, QFormLayout, QLineEdit, QComboBox, QDialogButtonBox
 )
@@ -22,7 +20,8 @@ from calibre import human_readable
 from calibre.ebooks.metadata.book.render import DEFAULT_AUTHOR_LINK
 from calibre.constants import ismacos, iswindows
 from calibre.ebooks.metadata.sources.prefs import msprefs
-from calibre.gui2 import default_author_link
+from calibre.gui2.custom_column_widgets import get_field_list as em_get_field_list
+from calibre.gui2 import default_author_link, icon_resource_manager, choose_save_file, choose_files
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget, CommaSeparatedList
 from calibre.gui2.preferences.look_feel_ui import Ui_Form
@@ -35,19 +34,11 @@ from calibre.gui2.book_details import get_field_list
 from calibre.gui2.dialogs.quickview import get_qv_field_list
 from calibre.gui2.preferences.coloring import EditRules
 from calibre.gui2.library.alternate_views import auto_height, CM_TO_INCH
+from calibre.gui2.widgets import BusyCursor
 from calibre.gui2.widgets2 import Dialog
 from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
 from calibre.utils.resources import set_data
-from polyglot.builtins import iteritems, unicode_type, map
-
-
-class BusyCursor(object):
-
-    def __enter__(self):
-        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
-
-    def __exit__(self, *args):
-        QApplication.restoreOverrideCursor()
+from polyglot.builtins import iteritems
 
 
 class DefaultAuthorLink(QWidget):  # {{{
@@ -118,6 +109,8 @@ class IdLinksRuleEdit(Dialog):
         title = _('Edit rule') if key else _('Create a new rule')
         Dialog.__init__(self, title=title, name='id-links-rule-editor', parent=parent)
         self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'https://example.com/{id}')
+        if self.size().height() < self.sizeHint().height():
+            self.resize(self.sizeHint())
 
     @property
     def rule(self):
@@ -135,7 +128,10 @@ class IdLinksRuleEdit(Dialog):
         self.nw = n = QLineEdit(self)
         l.addRow(_('&Name:'), n)
         la = QLabel(_(
-            'The template used to create the link. The placeholder {id} in the template will be replaced with the actual identifier value.'))
+            'The template used to create the link.'
+            ' The placeholder {0} in the template will be replaced'
+            ' with the actual identifier value. Use {1} to avoid the value'
+            ' being quoted.').format('{id}', '{id_unquoted}'))
         la.setWordWrap(True)
         l.addRow(la)
         self.template = t = QLineEdit(self)
@@ -176,14 +172,14 @@ class IdLinksEditor(Dialog):
             t.setItem(r, 1, QTableWidgetItem(val))
             t.setItem(r, 2, QTableWidgetItem(template))
         l.addWidget(t)
-        t.horizontalHeader().setSectionResizeMode(2, t.horizontalHeader().Stretch)
-        self.cb = b = QPushButton(QIcon(I('plus.png')), _('&Add rule'), self)
+        t.horizontalHeader().setSectionResizeMode(2, QHeaderView.ResizeMode.Stretch)
+        self.cb = b = QPushButton(QIcon.ic('plus.png'), _('&Add rule'), self)
         connect_lambda(b.clicked, self, lambda self: self.edit_rule())
         self.bb.addButton(b, QDialogButtonBox.ButtonRole.ActionRole)
-        self.rb = b = QPushButton(QIcon(I('minus.png')), _('&Remove rule'), self)
+        self.rb = b = QPushButton(QIcon.ic('minus.png'), _('&Remove rule'), self)
         connect_lambda(b.clicked, self, lambda self: self.remove_rule())
         self.bb.addButton(b, QDialogButtonBox.ButtonRole.ActionRole)
-        self.eb = b = QPushButton(QIcon(I('modified.png')), _('&Edit rule'), self)
+        self.eb = b = QPushButton(QIcon.ic('modified.png'), _('&Edit rule'), self)
         connect_lambda(b.clicked, self, lambda self: self.edit_rule(self.table.currentRow()))
         self.bb.addButton(b, QDialogButtonBox.ButtonRole.ActionRole)
         l.addWidget(self.bb)
@@ -205,7 +201,7 @@ class IdLinksEditor(Dialog):
         if r > -1:
             key, name, template = map(lambda c: self.table.item(r, c).text(), range(3))
         d = IdLinksRuleEdit(key, name, template, self)
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             if r < 0:
                 self.table.setRowCount(self.table.rowCount() + 1)
                 r = self.table.rowCount() - 1
@@ -255,19 +251,19 @@ class DisplayedFields(QAbstractListModel):  # {{{
             except:
                 pass
             if not name:
-                name = field
-            return name
+                return field
+            return f'{name} ({field})'
         if role == Qt.ItemDataRole.CheckStateRole:
             return Qt.CheckState.Checked if visible else Qt.CheckState.Unchecked
         if role == Qt.ItemDataRole.DecorationRole and field.startswith('#'):
-            return QIcon(I('column.png'))
+            return QIcon.ic('column.png')
         return None
 
     def toggle_all(self, show=True):
         for i in range(self.rowCount()):
             idx = self.index(i)
             if idx.isValid():
-                self.setData(idx, show, Qt.ItemDataRole.CheckStateRole)
+                self.setData(idx, Qt.CheckState.Checked if show else Qt.CheckState.Unchecked, Qt.ItemDataRole.CheckStateRole)
 
     def flags(self, index):
         ans = QAbstractListModel.flags(self, index)
@@ -276,7 +272,7 @@ class DisplayedFields(QAbstractListModel):  # {{{
     def setData(self, index, val, role):
         ret = False
         if role == Qt.ItemDataRole.CheckStateRole:
-            self.fields[index.row()][1] = bool(val)
+            self.fields[index.row()][1] = val in (Qt.CheckState.Checked, Qt.CheckState.Checked.value)
             self.changed = True
             ret = True
             self.dataChanged.emit(index, index)
@@ -324,6 +320,23 @@ def move_field_down(widget, model):
 # }}}
 
 
+class EMDisplayedFields(DisplayedFields):  # {{{
+    def __init__(self, db, parent=None):
+        DisplayedFields.__init__(self, db, parent)
+
+    def initialize(self, use_defaults=False, pref_data_override=None):
+        self.beginResetModel()
+        self.fields = [[x[0], x[1]] for x in
+                em_get_field_list(self.db, use_defaults=use_defaults, pref_data_override=pref_data_override)]
+        self.endResetModel()
+        self.changed = True
+
+    def commit(self):
+        if self.changed:
+            self.db.new_api.set_pref('edit_metadata_custom_columns_to_display', self.fields)
+# }}}
+
+
 class QVDisplayedFields(DisplayedFields):  # {{{
 
     def __init__(self, db, parent=None):
@@ -383,16 +396,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
 
     def genesis(self, gui):
         self.gui = gui
+        self.ui_style_available = True
         if not ismacos and not iswindows:
             self.label_widget_style.setVisible(False)
             self.opt_ui_style.setVisible(False)
+            self.ui_style_available = False
 
         db = gui.library_view.model().db
 
         r = self.register
 
         try:
-            self.icon_theme_title = json.loads(I('icon-theme.json', data=True))['name']
+            self.icon_theme_title = icon_resource_manager.user_theme_title
         except Exception:
             self.icon_theme_title = _('Default icons')
         self.icon_theme.setText(_('Icon theme: <b>%s</b>') % self.icon_theme_title)
@@ -401,10 +416,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.default_author_link = DefaultAuthorLink(self.default_author_link_container)
         self.default_author_link.changed_signal.connect(self.changed_signal)
         r('gui_layout', config, restart_required=True, choices=[(_('Wide'), 'wide'), (_('Narrow'), 'narrow')])
-        r('hidpi', gprefs, restart_required=True, choices=[(_('Automatic'), 'auto'), (_('On'), 'on'), (_('Off'), 'off')])
-        if ismacos:
-            self.opt_hidpi.setVisible(False), self.label_hidpi.setVisible(False)
         r('ui_style', gprefs, restart_required=True, choices=[(_('System default'), 'system'), (_('calibre style'), 'calibre')])
+        r('color_palette', gprefs, restart_required=True, choices=[(_('System default'), 'system'), (_('Light'), 'light'), (_('Dark'), 'dark')])
         r('book_list_tooltips', gprefs)
         r('dnd_merge', gprefs)
         r('wrap_toolbar_text', gprefs, restart_required=True)
@@ -476,6 +489,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('separate_cover_flow', config, restart_required=True)
         r('cb_fullscreen', gprefs)
         r('cb_preserve_aspect_ratio', gprefs)
+        r('cb_double_click_to_activate', gprefs)
 
         choices = [(_('Off'), 'off'), (_('Small'), 'small'),
             (_('Medium'), 'medium'), (_('Large'), 'large')]
@@ -510,6 +524,21 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                          key=lambda x:sort_key(x[0]))
         r('field_under_covers_in_grid', db.prefs, choices=choices)
 
+        choices = [(_('Default'), 'default'), (_('Compact metadata'), 'alt1'),
+                   (_('All on 1 tab'), 'alt2')]
+        r('edit_metadata_single_layout', gprefs,
+          choices=[(_('Default'), 'default'), (_('Compact metadata'), 'alt1'),
+                   (_('All on 1 tab'), 'alt2')])
+        r('edit_metadata_ignore_display_order', db.prefs)
+        r('edit_metadata_elision_point', gprefs,
+          choices=[(_('Left'), 'left'), (_('Middle'), 'middle'),
+                   (_('Right'), 'right')])
+        r('edit_metadata_elide_labels', gprefs)
+        r('edit_metadata_single_use_2_cols_for_custom_fields', gprefs)
+        r('edit_metadata_bulk_cc_label_length', gprefs)
+        r('edit_metadata_single_cc_label_length', gprefs)
+        r('edit_metadata_templates_only_F2_on_booklist', gprefs)
+
         self.current_font = self.initial_font = None
         self.change_font_button.clicked.connect(self.change_font)
 
@@ -522,6 +551,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         connect_lambda(self.df_down_button.clicked, self,
                 lambda self: move_field_down(self.field_display_order, self.display_model))
 
+        self.em_display_model = EMDisplayedFields(self.gui.current_db,
+                self.em_display_order)
+        self.em_display_model.dataChanged.connect(self.changed_signal)
+        self.em_display_order.setModel(self.em_display_model)
+        connect_lambda(self.em_up_button.clicked, self,
+                lambda self: move_field_up(self.em_display_order, self.em_display_model))
+        connect_lambda(self.em_down_button.clicked, self,
+                lambda self: move_field_down(self.em_display_order, self.em_display_model))
+        self.em_export_layout_button.clicked.connect(self.em_export_layout)
+        self.em_import_layout_button.clicked.connect(self.em_import_layout)
+        self.em_reset_layout_button.clicked.connect(self.em_reset_layout)
+
         self.qv_display_model = QVDisplayedFields(self.gui.current_db,
                 self.qv_display_order)
         self.qv_display_model.dataChanged.connect(self.changed_signal)
@@ -534,12 +575,12 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.edit_rules = EditRules(self.tabWidget)
         self.edit_rules.changed.connect(self.changed_signal)
         self.tabWidget.addTab(self.edit_rules,
-                QIcon(I('format-fill-color.png')), _('Column &coloring'))
+                QIcon.ic('format-fill-color.png'), _('Column &coloring'))
 
         self.icon_rules = EditRules(self.tabWidget)
         self.icon_rules.changed.connect(self.changed_signal)
         self.tabWidget.addTab(self.icon_rules,
-                QIcon(I('icon_choose.png')), _('Column &icons'))
+                QIcon.ic('icon_choose.png'), _('Column &icons'))
 
         self.grid_rules = EditRules(self.emblems_tab)
         self.grid_rules.changed.connect(self.changed_signal)
@@ -547,9 +588,10 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.emblems_tab.layout().addWidget(self.grid_rules)
 
         self.tabWidget.setCurrentIndex(0)
+        self.tabWidget.tabBar().setVisible(False)
         keys = [QKeySequence('F11', QKeySequence.SequenceFormat.PortableText), QKeySequence(
             'Ctrl+Shift+F', QKeySequence.SequenceFormat.PortableText)]
-        keys = [unicode_type(x.toString(QKeySequence.SequenceFormat.NativeText)) for x in keys]
+        keys = [str(x.toString(QKeySequence.SequenceFormat.NativeText)) for x in keys]
         self.fs_help_msg.setText(self.fs_help_msg.text()%(
             QKeySequence(QKeySequence.StandardKey.FullScreen).toString(QKeySequence.SequenceFormat.NativeText)))
         self.size_calculated.connect(self.update_cg_cache_size, type=Qt.ConnectionType.QueuedConnection)
@@ -566,7 +608,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         b.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         l.addWidget(b, 1, 1)
         b.clicked.connect(self.change_cover_grid_texture)
-        self.cover_grid_default_appearance_button = b = QPushButton(_('Restore &default appearance'), self)
+        self.cover_grid_default_appearance_button = b = QPushButton(_('Restore default &appearance'), self)
         b.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         l.addWidget(b, 2, 1)
         b.clicked.connect(self.restore_cover_grid_appearance)
@@ -584,18 +626,63 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.css_highlighter = get_highlighter('css')()
         self.css_highlighter.apply_theme(get_theme(None))
         self.css_highlighter.set_document(self.opt_book_details_css.document())
+        for i in range(self.tabWidget.count()):
+            self.sections_view.addItem(QListWidgetItem(self.tabWidget.tabIcon(i), self.tabWidget.tabText(i).replace('&', '')))
+        self.sections_view.setCurrentRow(self.tabWidget.currentIndex())
+        self.sections_view.currentRowChanged.connect(self.tabWidget.setCurrentIndex)
+        self.sections_view.setMaximumWidth(self.sections_view.sizeHintForColumn(0) + 16)
+        self.sections_view.setSpacing(4)
+        self.sections_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self.tabWidget.currentWidget().setFocus(Qt.FocusReason.OtherFocusReason)
+        self.opt_ui_style.currentIndexChanged.connect(self.update_color_palette_state)
+
+    def update_color_palette_state(self):
+        if self.ui_style_available:
+            enabled = self.opt_ui_style.currentData() == 'calibre'
+            self.opt_color_palette.setEnabled(enabled)
+            self.opt_color_palette_label.setEnabled(enabled)
+
+    def em_export_layout(self):
+        filename = choose_save_file(self, 'em_import_export_field_list',
+                _('Save column list to file'),
+                filters=[(_('Column list'), ['json'])])
+        if filename:
+            try:
+                with open(filename, 'w') as f:
+                    json.dump(self.em_display_model.fields, f, indent=1)
+            except Exception as err:
+                error_dialog(self, _('Export field layout'),
+                             _('<p>Could not write field list. Error:<br>%s')%err, show=True)
+
+    def em_import_layout(self):
+        filename = choose_files(self, 'em_import_export_field_list',
+                _('Load column list from file'),
+                filters=[(_('Column list'), ['json'])])
+        if filename:
+            try:
+                with open(filename[0]) as f:
+                    fields = json.load(f)
+                self.em_display_model.initialize(pref_data_override=fields)
+                self.changed_signal.emit()
+            except Exception as err:
+                error_dialog(self, _('Import layout'),
+                             _('<p>Could not read field list. Error:<br>%s')%err, show=True)
+
+    def em_reset_layout(self):
+        self.em_display_model.initialize(use_defaults=True)
+        self.changed_signal.emit()
 
     def choose_icon_theme(self):
         from calibre.gui2.icon_theme import ChooseTheme
         d = ChooseTheme(self)
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.commit_icon_theme = d.commit_changes
             self.icon_theme_title = d.new_theme_title or _('Default icons')
             self.icon_theme.setText(_('Icon theme: <b>%s</b>') % self.icon_theme_title)
             self.changed_signal.emit()
 
     def edit_id_link_rules(self):
-        if IdLinksEditor(self).exec_() == QDialog.DialogCode.Accepted:
+        if IdLinksEditor(self).exec() == QDialog.DialogCode.Accepted:
             self.changed_signal.emit()
 
     @property
@@ -630,7 +717,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def edit_cb_title_template(self):
         t = TemplateDialog(self, self.opt_cover_browser_title_template.text(), fm=self.gui.current_db.field_metadata)
         t.setWindowTitle(_('Edit template for caption'))
-        if t.exec_():
+        if t.exec():
             self.opt_cover_browser_title_template.setText(t.rule[1])
 
     def initialize(self):
@@ -643,13 +730,17 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.current_font = self.initial_font = font
         self.update_font_display()
         self.display_model.initialize()
+        self.em_display_model.initialize()
         self.qv_display_model.initialize()
         db = self.gui.current_db
+        mi = []
         try:
-            idx = self.gui.library_view.currentIndex().row()
-            mi = db.get_metadata(idx, index_is_id=False)
+            rows = self.gui.current_view().selectionModel().selectedRows()
+            for row in rows:
+                if row.isValid():
+                    mi.append(db.new_api.get_proxy_metadata(db.data.index_to_id(row.row())))
         except:
-            mi=None
+            pass
         self.edit_rules.initialize(db.field_metadata, db.prefs, mi, 'column_color_rules')
         self.icon_rules.initialize(db.field_metadata, db.prefs, mi, 'column_icon_rules')
         self.grid_rules.initialize(db.field_metadata, db.prefs, mi, 'cover_grid_icon_rules')
@@ -660,6 +751,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.opt_book_details_css.setPlainText(P('templates/book_details.css', data=True).decode('utf-8'))
         self.opt_book_details_css.blockSignals(False)
         self.tb_focus_label.setVisible(self.opt_tag_browser_allow_keyboard_focus.isChecked())
+        self.update_color_palette_state()
 
     def open_cg_cache(self):
         open_local_file(self.gui.grid_view.thumbnail_cache.location)
@@ -701,6 +793,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.changed_signal.emit()
             self.update_font_display()
         self.display_model.restore_defaults()
+        self.em_display_model.restore_defaults()
         self.qv_display_model.restore_defaults()
         self.edit_rules.clear()
         self.icon_rules.clear()
@@ -727,7 +820,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def change_cover_grid_texture(self):
         from calibre.gui2.preferences.texture_chooser import TextureChooser
         d = TextureChooser(parent=self, initial=self.cg_bg_widget.btex)
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.set_cg_texture(d.texture)
             self.changed_signal.emit()
 
@@ -745,17 +838,17 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def update_font_display(self):
         font = self.build_font_obj()
         fi = QFontInfo(font)
-        name = unicode_type(fi.family())
+        name = str(fi.family())
 
         self.font_display.setFont(font)
         self.font_display.setText(name + ' [%dpt]'%fi.pointSize())
 
     def change_font(self, *args):
         fd = QFontDialog(self.build_font_obj(), self)
-        if fd.exec_() == QDialog.DialogCode.Accepted:
+        if fd.exec() == QDialog.DialogCode.Accepted:
             font = fd.selectedFont()
             fi = QFontInfo(font)
-            self.current_font = [unicode_type(fi.family()), fi.pointSize(),
+            self.current_font = [str(fi.family()), fi.pointSize(),
                     fi.weight(), fi.italic(), font.stretch()]
             self.update_font_display()
             self.changed_signal.emit()
@@ -771,6 +864,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 QApplication.setFont(self.font_display.font())
                 rr = True
             self.display_model.commit()
+            self.em_display_model.commit()
             self.qv_display_model.commit()
             self.edit_rules.commit(self.gui.current_db.prefs)
             self.icon_rules.commit(self.gui.current_db.prefs)
@@ -779,7 +873,6 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             gprefs['cover_grid_texture'] = self.cg_bg_widget.btex
             if self.commit_icon_theme is not None:
                 self.commit_icon_theme()
-                rr = True
             gprefs['default_author_link'] = self.default_author_link.value
             bcss = self.opt_book_details_css.toPlainText().encode('utf-8')
             defcss = P('templates/book_details.css', data=True, allow_user_override=False)
@@ -798,9 +891,11 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         gui.tags_view.reread_collapse_parameters()
         gui.library_view.refresh_book_details(force=True)
         gui.library_view.refresh_grid()
+        gui.library_view.refresh_composite_edit()
         gui.library_view.set_row_header_visibility()
         gui.cover_flow.setShowReflections(gprefs['cover_browser_reflections'])
         gui.cover_flow.setPreserveAspectRatio(gprefs['cb_preserve_aspect_ratio'])
+        gui.cover_flow.setActivateOnDoubleClick(gprefs['cb_double_click_to_activate'])
         gui.update_cover_flow_subtitle_font()
         gui.cover_flow.template_inited = False
         for view in 'library memory card_a card_b'.split():
