@@ -1,19 +1,17 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from PyQt5.Qt import QAbstractListModel, Qt, QIcon
+from qt.core import QAbstractListModel, Qt, QIcon, QItemSelectionModel
 
 from calibre import force_unicode
 from calibre.gui2.preferences.toolbar_ui import Ui_Form
 from calibre.gui2 import gprefs, warning_dialog, error_dialog
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget, AbortCommit
 from calibre.utils.icu import primary_sort_key
-from polyglot.builtins import unicode_type
 
 
 def sort_key_for_action(ac):
@@ -25,7 +23,7 @@ def sort_key_for_action(ac):
         return primary_sort_key('')
 
 
-class FakeAction(object):
+class FakeAction:
 
     def __init__(self, name, gui_name, icon, tooltip=None,
             dont_add_to=frozenset(), dont_remove_from=frozenset()):
@@ -63,13 +61,13 @@ class BaseModel(QAbstractListModel):
     def data(self, index, role):
         row = index.row()
         action = self._data[row].action_spec
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             text = action[0]
             text = text.replace('&', '')
             if text == _('%d books'):
                 text = _('Choose library')
             return (text)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             if hasattr(self._data[row], 'qaction'):
                 icon = self._data[row].qaction.icon()
                 if not icon.isNull():
@@ -77,8 +75,8 @@ class BaseModel(QAbstractListModel):
             ic = action[1]
             if ic is None:
                 ic = 'blank.png'
-            return (QIcon(I(ic)))
-        if role == Qt.ToolTipRole and action[2] is not None:
+            return (QIcon.ic(ic))
+        if role == Qt.ItemDataRole.ToolTipRole and action[2] is not None:
             return (action[2])
         return None
 
@@ -259,6 +257,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             ]
 
     def genesis(self, gui):
+        self.all_actions.doubleClicked.connect(self.add_single_action)
+        self.current_actions.doubleClicked.connect(self.remove_single_action)
         self.models = {}
         self.what.addItem(_('Click to choose toolbar or menu to customize'),
                 'blank')
@@ -268,7 +268,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             current_model = CurrentModel(key, gui)
             self.models[key] = (all_model, current_model)
         self.what.setCurrentIndex(0)
-        self.what.currentIndexChanged[int].connect(self.what_changed)
+        self.what.currentIndexChanged.connect(self.what_changed)
         self.what_changed(0)
 
         self.add_action_button.clicked.connect(self.add_action)
@@ -281,15 +281,15 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.current_actions.entered.connect(self.current_entered)
 
     def all_entered(self, index):
-        tt = self.all_actions.model().data(index, Qt.ToolTipRole) or ''
+        tt = self.all_actions.model().data(index, Qt.ItemDataRole.ToolTipRole) or ''
         self.help_text.setText(tt)
 
     def current_entered(self, index):
-        tt = self.current_actions.model().data(index, Qt.ToolTipRole) or ''
+        tt = self.current_actions.model().data(index, Qt.ItemDataRole.ToolTipRole) or ''
         self.help_text.setText(tt)
 
     def what_changed(self, idx):
-        key = unicode_type(self.what.itemData(idx) or '')
+        key = str(self.what.itemData(idx) or '')
         if key == 'blank':
             self.actions_widget.setVisible(False)
             self.spacer_widget.setVisible(True)
@@ -300,13 +300,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.current_actions.setModel(self.models[key][1])
 
     def add_action(self, *args):
-        x = self.all_actions.selectionModel().selectedIndexes()
-        names = self.all_actions.model().names(x)
+        self._add_action(self.all_actions.selectionModel().selectedIndexes())
+
+    def add_single_action(self, index):
+        self._add_action([index])
+
+    def _add_action(self, indices):
+        names = self.all_actions.model().names(indices)
         if names:
             not_added = self.current_actions.model().add(names)
             ns = {y.name for y in not_added}
             added = set(names) - ns
-            self.all_actions.model().remove(x, added)
+            self.all_actions.model().remove(indices, added)
             if not_added:
                 warning_dialog(self, _('Cannot add'),
                         _('Cannot add the actions %s to this location') %
@@ -319,10 +324,15 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.changed_signal.emit()
 
     def remove_action(self, *args):
-        x = self.current_actions.selectionModel().selectedIndexes()
-        names = self.current_actions.model().names(x)
+        self._remove_action(self.current_actions.selectionModel().selectedIndexes())
+
+    def remove_single_action(self, index):
+        self._remove_action([index])
+
+    def _remove_action(self, indices):
+        names = self.current_actions.model().names(indices)
         if names:
-            not_removed = self.current_actions.model().remove(x)
+            not_removed = self.current_actions.model().remove(indices)
             ns = {y.name for y in not_removed}
             removed = set(names) - ns
             self.all_actions.model().add(removed)
@@ -343,10 +353,10 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             idx_map = m.move_many(x, delta)
             newci = idx_map.get(i)
             if newci is not None:
-                sm.setCurrentIndex(newci, sm.ClearAndSelect)
+                sm.setCurrentIndex(newci, QItemSelectionModel.SelectionFlag.ClearAndSelect)
             sm.clear()
             for idx in idx_map.values():
-                sm.select(idx, sm.Select)
+                sm.select(idx, QItemSelectionModel.SelectionFlag.Select)
             self.changed_signal.emit()
 
     def commit(self):
