@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
 import json
@@ -7,17 +6,19 @@ import regex
 from collections import Counter, OrderedDict
 from html import escape
 from qt.core import (
-    QCheckBox, QComboBox, QFont, QHBoxLayout, QIcon, QLabel, Qt, QToolButton,
-    QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+    QAbstractItemView, QCheckBox, QComboBox, QFont, QHBoxLayout, QIcon, QLabel, Qt,
+    QToolButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
 )
 from threading import Thread
 
 from calibre.ebooks.conversion.search_replace import REGEX_FLAGS
 from calibre.gui2 import warning_dialog
+from calibre.gui2.gestures import GestureManager
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.viewer.config import vprefs
 from calibre.gui2.viewer.web_view import get_data, get_manifest
 from calibre.gui2.viewer.widgets import ResultsDelegate, SearchBox
+from calibre.utils.icu import primary_collator_without_punctuation
 from polyglot.builtins import iteritems
 from polyglot.functools import lru_cache
 from polyglot.queue import Queue
@@ -105,7 +106,7 @@ class Search:
                 if self.mode == 'word':
                     words = []
                     for part in expr.split():
-                        words.append(r'\b{}\b'.format(text_to_regex(part)))
+                        words.append(fr'\b{text_to_regex(part)}\b')
                     expr = r'\s+'.join(words)
                 else:
                     expr = text_to_regex(expr)
@@ -170,9 +171,12 @@ def searchable_text_for_name(name):
     ans = []
     serialized_data = json.loads(get_data(name)[0])
     stack = []
+    removed_tails = []
     for child in serialized_data['tree']['c']:
         if child.get('n') == 'body':
             stack.append(child)
+            # the JS code does not add the tail of body tags to flat text
+            removed_tails.append((child.pop('l', None), child))
     ignore_text = {'script', 'style', 'title'}
     text_pos = 0
     anchor_offset_map = OrderedDict()
@@ -201,6 +205,9 @@ def searchable_text_for_name(name):
             stack.append(tail)
         if children:
             stack.extend(reversed(children))
+    for (tail, body) in removed_tails:
+        if tail is not None:
+            body['l'] = tail
     return ''.join(ans), anchor_offset_map
 
 
@@ -305,11 +312,22 @@ def toc_nodes_for_search_result(sr):
 
 def search_in_name(name, search_query, ctx_size=75):
     raw = searchable_text_for_name(name)[0]
-    for match in search_query.regex.finditer(raw):
-        start, end = match.span()
+
+    if search_query.mode == 'regex' or search_query.case_sensitive:
+        def miter():
+            for match in search_query.regex.finditer(raw):
+                yield match.span()
+    else:
+        spans = []
+        miter = lambda: spans
+        if raw:
+            a = lambda s, l: spans.append((s, s + l))
+            primary_collator_without_punctuation().find_all(search_query.text, raw, a, search_query.mode == 'word')
+
+    for (start, end) in miter():
         before = raw[max(0, start-ctx_size):start]
         after = raw[end:end+ctx_size]
-        yield before, match.group(), after, start
+        yield before, raw[start:end], after, start
 
 
 class SearchInput(QWidget):  # {{{
@@ -329,7 +347,7 @@ class SearchInput(QWidget):  # {{{
 
         self.search_box = sb = SearchBox(self)
         self.panel_name = panel_name
-        sb.initialize('viewer-{}-panel-expression'.format(panel_name))
+        sb.initialize(f'viewer-{panel_name}-panel-expression')
         sb.item_selected.connect(self.saved_search_selected)
         sb.history_saved.connect(self.history_saved)
         sb.history_cleared.connect(self.history_cleared)
@@ -340,14 +358,14 @@ class SearchInput(QWidget):  # {{{
         self.next_button = nb = QToolButton(self)
         h.addWidget(nb)
         nb.setFocusPolicy(Qt.FocusPolicy.NoFocus)
-        nb.setIcon(QIcon(I('arrow-down.png')))
+        nb.setIcon(QIcon.ic('arrow-down.png'))
         nb.clicked.connect(self.find_next)
         nb.setToolTip(_('Find next match'))
 
         self.prev_button = nb = QToolButton(self)
         h.addWidget(nb)
         nb.setFocusPolicy(Qt.FocusPolicy.NoFocus)
-        nb.setIcon(QIcon(I('arrow-up.png')))
+        nb.setIcon(QIcon.ic('arrow-up.png'))
         nb.clicked.connect(self.find_previous)
         nb.setToolTip(_('Find previous match'))
 
@@ -361,48 +379,50 @@ class SearchInput(QWidget):  # {{{
         qt.addItem(_('Regex'), 'regex')
         qt.setToolTip('<p>' + _(
             'Choose the type of search: <ul>'
-            '<li><b>Contains</b> will search for the entered text anywhere.'
-            '<li><b>Whole words</b> will search for whole words that equal the entered text.'
+            '<li><b>Contains</b> will search for the entered text anywhere. It will ignore punctuation,'
+            ' spaces and accents, unless Case sensitive searching is enabled.'
+            '<li><b>Whole words</b> will search for whole words that equal the entered text. As with'
+            ' "Contains" searches punctuation and accents are ignored.'
             '<li><b>Regex</b> will interpret the text as a regular expression.'
         ))
-        qt.setCurrentIndex(qt.findData(vprefs.get('viewer-{}-mode'.format(self.panel_name), 'normal') or 'normal'))
+        qt.setCurrentIndex(qt.findData(vprefs.get(f'viewer-{self.panel_name}-mode', 'normal') or 'normal'))
         qt.currentIndexChanged.connect(self.save_search_type)
         h.addWidget(qt)
 
         self.case_sensitive = cs = QCheckBox(_('&Case sensitive'), self)
         cs.setFocusPolicy(Qt.FocusPolicy.NoFocus)
-        cs.setChecked(bool(vprefs.get('viewer-{}-case-sensitive'.format(self.panel_name), False)))
+        cs.setChecked(bool(vprefs.get(f'viewer-{self.panel_name}-case-sensitive', False)))
         cs.stateChanged.connect(self.save_search_type)
         h.addWidget(cs)
 
         self.return_button = rb = QToolButton(self)
-        rb.setIcon(QIcon(I('back.png')))
+        rb.setIcon(QIcon.ic('back.png'))
         rb.setToolTip(_('Go back to where you were before searching'))
         rb.clicked.connect(self.go_back)
         h.addWidget(rb)
 
     def history_saved(self, new_text, history):
         if new_text:
-            sss = vprefs.get('saved-{}-settings'.format(self.panel_name)) or {}
+            sss = vprefs.get(f'saved-{self.panel_name}-settings') or {}
             sss[new_text] = {'case_sensitive': self.case_sensitive.isChecked(), 'mode': self.query_type.currentData()}
             history = frozenset(history)
             sss = {k: v for k, v in iteritems(sss) if k in history}
-            vprefs['saved-{}-settings'.format(self.panel_name)] = sss
+            vprefs[f'saved-{self.panel_name}-settings'] = sss
 
     def history_cleared(self):
-        vprefs['saved-{}-settings'.format(self.panel_name)] = {}
+        vprefs[f'saved-{self.panel_name}-settings'] = {}
 
     def save_search_type(self):
         text = self.search_box.currentText()
         if text and not self.ignore_search_type_changes:
-            sss = vprefs.get('saved-{}-settings'.format(self.panel_name)) or {}
+            sss = vprefs.get(f'saved-{self.panel_name}-settings') or {}
             sss[text] = {'case_sensitive': self.case_sensitive.isChecked(), 'mode': self.query_type.currentData()}
-            vprefs['saved-{}-settings'.format(self.panel_name)] = sss
+            vprefs[f'saved-{self.panel_name}-settings'] = sss
 
     def saved_search_selected(self):
         text = self.search_box.currentText()
         if text:
-            s = (vprefs.get('saved-{}-settings'.format(self.panel_name)) or {}).get(text)
+            s = (vprefs.get(f'saved-{self.panel_name}-settings') or {}).get(text)
             if s:
                 self.ignore_search_type_changes = True
                 if 'case_sensitive' in s:
@@ -423,8 +443,8 @@ class SearchInput(QWidget):  # {{{
             )
 
     def emit_search(self, backwards=False):
-        vprefs['viewer-{}-case-sensitive'.format(self.panel_name)] = self.case_sensitive.isChecked()
-        vprefs['viewer-{}-mode'.format(self.panel_name)] = self.query_type.currentData()
+        vprefs[f'viewer-{self.panel_name}-case-sensitive'] = self.case_sensitive.isChecked()
+        vprefs[f'viewer-{self.panel_name}-mode'] = self.query_type.currentData()
         sq = self.search_query(backwards)
         if sq is not None:
             self.do_search.emit(sq)
@@ -435,9 +455,16 @@ class SearchInput(QWidget):  # {{{
     def find_previous(self):
         self.emit_search(backwards=True)
 
-    def focus_input(self, text=None):
+    def focus_input(self, text=None, search_type=None, case_sensitive=None):
         if text and hasattr(text, 'rstrip'):
             self.search_box.setText(text)
+        if search_type is not None:
+            idx = self.query_type.findData(search_type)
+            if idx < 0:
+                idx = self.query_type.findData('normal')
+            self.query_type.setCurrentIndex(idx)
+        if case_sensitive is not None:
+            self.case_sensitive.setChecked(bool(case_sensitive))
         self.search_box.setFocus(Qt.FocusReason.OtherFocusReason)
         le = self.search_box.lineEdit()
         le.end(False)
@@ -458,14 +485,23 @@ class Results(QTreeWidget):  # {{{
         self.delegate = ResultsDelegate(self)
         self.setItemDelegate(self.delegate)
         self.itemClicked.connect(self.item_activated)
-        self.blank_icon = QIcon(I('blank.png'))
-        self.not_found_icon = QIcon(I('dialog_warning.png'))
+        self.blank_icon = QIcon.ic('blank.png')
+        self.not_found_icon = QIcon.ic('dialog_warning.png')
         self.currentItemChanged.connect(self.current_item_changed)
         self.section_font = QFont(self.font())
         self.section_font.setItalic(True)
         self.section_map = {}
         self.search_results = []
         self.item_map = {}
+        self.gesture_manager = GestureManager(self)
+        self.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
+
+    def viewportEvent(self, ev):
+        if hasattr(self, 'gesture_manager'):
+            ret = self.gesture_manager.handle_event(ev)
+            if ret is not None:
+                return ret
+        return super().viewportEvent(ev)
 
     def current_item_changed(self, current, previous):
         if current is not None:
@@ -638,8 +674,8 @@ class SearchPanel(QWidget):  # {{{
     def update_hidden_message(self):
         self.hidden_message.setVisible(self.results.current_result_is_hidden)
 
-    def focus_input(self, text=None):
-        self.search_input.focus_input(text)
+    def focus_input(self, text=None, search_type=None, case_sensitive=None):
+        self.search_input.focus_input(text, search_type, case_sensitive)
 
     def search_cleared(self):
         self.results.clear_all_results()
@@ -754,7 +790,7 @@ class SearchPanel(QWidget):  # {{{
 
     def show_no_results_found(self):
         msg = _('No matches were found for:')
-        warning_dialog(self, _('No matches found'), msg + '  <b>{}</b>'.format(self.current_search.text), show=True)
+        warning_dialog(self, _('No matches found'), msg + f'  <b>{self.current_search.text}</b>', show=True)
 
     def keyPressEvent(self, ev):
         if ev.key() == Qt.Key.Key_Escape:
